package api

import (
	"net/http"

	webpush "github.com/SherClockHolmes/webpush-go"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"

	models "dev.sum7.eu/genofire/push-pubsub/db"
)

func Use(r *gin.RouterGroup, db *models.Database) {
	g := r.Group("/v1")
	{
		g.GET("/status", func(c *gin.Context) {
			c.JSONP(http.StatusOK, gin.H{
				"status":           "running",
				"vapid_public_key": db.VAPID.PublicKey,
			})
		})
		g.POST("/push/subscribe", func(c *gin.Context) {
			data := webpush.Subscription{}
			if err := c.BindJSON(&data); err != nil {
				c.JSON(http.StatusBadRequest, err.Error())
				return
			}
			if data.Endpoint == "" || data.Keys.Auth == "" || data.Keys.P256dh == "" {
				c.JSON(http.StatusBadRequest, "endpoint or/and keys not filled")
				return
			}
			id, err := uuid.NewRandom()
			if err != nil {
				c.JSON(http.StatusBadRequest, err.Error())
				return
			}
			db.Sessions[id] = models.NewSessionPushNotification(data)
			c.JSON(http.StatusOK, id)

		})
		g.POST("/push/notify", func(c *gin.Context) {
			data := make(map[string]interface{})
			if err := c.BindJSON(&data); err != nil {
				c.JSON(http.StatusBadRequest, err.Error())
				return
			}
			db.SendAll(data)
			c.JSON(http.StatusOK, data)
		})
	}
}
