package db

import (
	"encoding/json"

	webpush "github.com/SherClockHolmes/webpush-go"
)

type VAPIDPushService struct {
	PublicKey  string `toml:"public_key"`
	PrivateKey string `toml:"private_key"`
	Subscriber string `toml:"subscriber"`
	TTL        int    `toml:"ttl"`
}

func (vc *VAPIDPushService) Generate() (bool, error) {
	var err error
	if vc.Subscriber == "" {
		vc.Subscriber = "example@example.com"
	}
	if vc.PrivateKey == "" || vc.PublicKey == "" {
		vc.PrivateKey, vc.PublicKey, err = webpush.GenerateVAPIDKeys()
		return true, err
	}
	return false, nil
}

func (vc *VAPIDPushService) SendNotification(s *webpush.Subscription, data interface{}) error {
	dataBytes, _ := json.Marshal(data)

	resp, err := webpush.SendNotification(dataBytes, s, &webpush.Options{
		Subscriber:      vc.Subscriber,
		VAPIDPublicKey:  vc.PublicKey,
		VAPIDPrivateKey: vc.PrivateKey,
		TTL:             vc.TTL,
	})
	if err != nil {
		return err
	}
	defer resp.Body.Close()
	return nil
}

type SessionPushNotification struct {
	Endpoint   string `json:"endpoint"`
	KeysAuth   string `json:"-"`
	KeysP256dh string `json:"-"`
}

func NewSessionPushNotification(origin webpush.Subscription) SessionPushNotification {
	return SessionPushNotification{
		Endpoint:   origin.Endpoint,
		KeysAuth:   origin.Keys.Auth,
		KeysP256dh: origin.Keys.P256dh,
	}
}

func (pn *SessionPushNotification) ToVAPID() *webpush.Subscription {
	return &webpush.Subscription{
		Endpoint: pn.Endpoint,
		Keys: webpush.Keys{
			Auth:   pn.KeysAuth,
			P256dh: pn.KeysP256dh,
		},
	}
}
